package client;

import java.net.*;
import java.io.*;

/**
 * A client that registers itself periodically with the server. In
 * return, the server also sends him a list of all the registered
 * clients. This uses a TCP/IP connection throughout the entire
 * communcation.
 */
public class RegistrationClient extends Thread {
	/* Networking variables */
	protected Socket socket = null; // TCP connection with the server
	protected InputStream in = null; // The stream to write on
	protected OutputStream out = null; // The stream to read from

	/* Command line parameters */
	protected InetAddress serverAddress; // The server address (dcg or bluebottle)
	protected String name; // Your own (real) name
	protected int port; // Your free port

	/**
	 * Creates a client that registers itself as listening to the
	 * given port.
	 */
	public RegistrationClient(InetAddress serverAddress, String name, int port) {
		super("IM RegistrationClient");

		this.serverAddress = serverAddress;
		this.name = name;
		this.port = port;
	}

	/**
	 * Invoked automatically by the jvm before deleting the object ("destructor").
	 */
	protected void finalize() throws IOException {
		closeConnection();
	}

	/**
	 * Opens the connection with the server.
	 */
	protected void openConnection() throws IOException {
		socket = new Socket(serverAddress, NetworkUtilities.SERVERPORT);
		in = socket.getInputStream();
		out = socket.getOutputStream();
	}

	/**
	 * Closes the connection with the server.
	 */
	protected void closeConnection() throws IOException {
		socket.close();
	}

	public void run() {
		while (true) {
			try {
				/* Open connection */
				openConnection();

				/* Registers itself with the server */
				registerUser();

				/* Receives and prints the user list */
				receiveUserList();

				/* Close connection */
				closeConnection();

				/* sleep some time -> should be in a variable... (config?) */
				Thread.sleep(60 * 1000);
			} catch (IOException ioe) {
				System.out.println("Oops, we caused an IOError. Stopping.");
				ioe.printStackTrace();
				break;
			} catch (InterruptedException ie) {
				/* Interrupted from sleeping -> wake up and do your work */
				continue;
			} catch (Exception e) {
				System.out.println("Something went wrong. Stopping.");
				e.printStackTrace();
				break;
			}
		}
		System.out.println("Registration client done.");
		System.exit(0);
	}

	// -----------------------------------------------------------------------------

	/**
		 * Sends the following information to register with the server (number
		 * of bytes given in parenthesis):
	 * type of message: REGISTER (1), own port (2), length of name (1),
	 * own name (var <= 255).
	 */
	protected void registerUser() throws IOException {
		out.write(NetworkUtilities.REGISTER); // write REGISTER command
		NetworkUtilities.write16(out, port); // write port, using write16
		out.write(name.length()); // write length of name
		out.write(name.getBytes(), 0, name.length()); // write name as byte array
		out.flush(); // flush the stream
	}

	/**
	 * The server sends the list of all active clients (number of bytes given
	 * in parenthesis):
	 * type of message: USERLIST (1), number of users (2), list of users,
	 * where each user is defined as: length of name (1), name (var <= 255),
	 * ip (4), port (2). The ip is given with most significant byte first, ie.
	 * the address 129.132.130.158 is sent/received with 129 as first byte,
	 * 132 second...
	 */
	protected void receiveUserList() throws IOException {
		byte type = (byte)in.read();
		if (type != NetworkUtilities.USERLIST) { // compare received and expected type
			System.out.println("Unrecognized protocol message.");
			throw new IOException("Unrecognized protocol message.");
		}
		int number = NetworkUtilities.read16(in);
		User[] users = new User[number];
		for (int i = 0; i < number; i++) {
			int nameLength = in.read(); // read the length of the name
			byte[] nameBuffer = new byte[nameLength];
			in.read(nameBuffer); // read the name as byte array
			String name = new String(nameBuffer); // convert byte array to string
			byte[] ip = new byte[4];
			in.read(ip); // read ip as byte array
			int port = NetworkUtilities.read16(in); // read the port number
			users[i] = new User(name, ip, port);
		}

		InstantMessenger.IM.setUsers(users);
	}
}